/*
 * Created on Aug 2, 2004
 *
 * To change this generated comment go to 
 * Window>Preferences>Java>Code Generation>Code Template
 */
package org.programmingbasics.iu.game;

import java.awt.Graphics2D;
import java.awt.geom.Rectangle2D;
import java.util.HashSet;
import java.util.Iterator;

/**
 * @author The World
 */
public class ArrowObject extends GameObjectWithPos {
   static RotatingSprite arrowImg;
   double angle;
   long startTime = -1;
   long lastTime = -1;

   static final int ARROW_DAMAGE = 10;
   static final long MAX_LIFETIME = 2000;
   static final double MOVE_SIZE = 0.2;
   
   /**
    * 
    */
   public ArrowObject(GameEngine e, double x, double y, double angle, long now) {
      super();
      // TODO: NOT SYNCHRONIZED!
      if (arrowImg == null)
         arrowImg = new RotatingSprite(e.loadImage("arrow.png"), 6, 0);
      this.angle = angle;
      this.x = x;
      this.y = y;
      this.startTime = now;
      this.lastTime = now;
   }

   void draw(GameEngine e, Graphics2D g)
   {
      e.drawSprite(arrowImg, x, y, angle - Math.PI / 2);
   }

   boolean checkCollision(CollisionChecker check)
   {
      return check.checkCircleCollision(x, y, 0);
   }
   
   void boundingBox(java.awt.geom.Rectangle2D r)
   {
      r.setRect(x, y, 0, 0);
   }

   void doMove(long now, GameMap map) 
   {
      if (now-startTime > MAX_LIFETIME)
      {
         map.removeGameObject(this);
         return;
      }
         
      long elapsed = now - lastTime;
      long timeSpentMoving = elapsed;
      lastTime = now;
      
      double newx = x + Math.cos(angle) * timeSpentMoving * MOVE_SIZE;
      double newy = y + Math.sin(angle) * timeSpentMoving * MOVE_SIZE;

      HashSet collided = new HashSet();
      map.findAllCollisions(new LineCollisionChecker(this, x, y, newx, newy), collided);

      Rectangle2D r = new Rectangle2D.Double();
      boundingBox(r);
      map.removeFromCollisionRegions(this, r);
      x = newx;
      y = newy;
      boundingBox(r);
      map.addToCollisionRegions(this, r);
      
      if (!collided.isEmpty())
      {
         Iterator it = collided.iterator();
         while (it.hasNext())
         {
            GameObject obj = (GameObject)it.next(); 
            if (obj.isPlayer())
            {
               obj.doDamage(ARROW_DAMAGE);
            }
         }
         map.removeGameObject(this);
         return;
      }
   }
}
