/*
 * Created on Aug 2, 2004
 *
 * To change this generated comment go to 
 * Window>Preferences>Java>Code Generation>Code Template
 */
package org.programmingbasics.iu.game;

import java.awt.Graphics2D;
import java.awt.geom.Rectangle2D;
import java.util.HashSet;
import java.util.Iterator;

import javax.sound.sampled.Clip;

/**
 * @author The World
 */
public class EnemyArcher extends GameObjectWithPos {
   static final double CollisionCircleSize = 50;
   static RotatingSprite standImg, leftImg, rightImg, extendImg;
   static SoundClip shootClip;
   int hitpoints = 10;
   double moveAngle;
   double attackAngle;
   boolean isAiming = false;

   static final long AIM_TIME = 1000;
   static final long ATTACK_TIME = 1000;
   static final long LEG_STEP_TIME = 500;
   static final long TURN_TIME = 100;
   static final double TURN_ANGLE = 0.8; 
   static final int MoveAmount = 30;
   
   static final int LEG_CENTER = 0;
   static final int LEG_LEFT = 1;
   static final int LEG_CENTER_RIGHT = 2;
   static final int LEG_RIGHT = 3;
   static final int LEG_NUM = 4;
   boolean isAttacking = false;
   int pos = LEG_CENTER;
   double attackDist = 300;
   
   long lastMoveTime = -1;
   GameEngine cachedEngine;
   
   /**
    * 
    */
   public EnemyArcher(GameEngine e, double x, double y) {
      super();
      cachedEngine = e;
      // TODO: NOT SYNCHRONIZED!
      if (standImg == null) {
         standImg = new RotatingSprite(e.loadImage("enemystand.png"), 68, 103);
         leftImg = new RotatingSprite(e.loadImage("enemyleft.png"), 68, 103);
         rightImg = new RotatingSprite(e.loadImage("enemyright.png"), 68, 103);
         extendImg = new RotatingSprite(e.loadImage("enemyextend.png"), 68, 103);
         shootClip = e.loadSound("flut.wav");
      }
         this.x = x;
      this.y = y;
   }

   void draw(GameEngine e, Graphics2D g)
   {
      RotatingSprite img = standImg;
      if (pos == LEG_LEFT) 
         img = leftImg;
      else if (pos == LEG_RIGHT)
         img = rightImg;
      if (isAttacking)
      {
         if (isAiming) img = standImg; else img = extendImg;
         e.drawSprite(img, x, y, attackAngle - Math.PI / 2);
      }
      else
         e.drawSprite(img, x, y, moveAngle - Math.PI / 2);
//      g.drawOval((int)x - 50, (int)y - 50, (int)100, (int)100);
   }

   boolean checkCollision(CollisionChecker check)
   {
      return check.checkCircleCollision(x, y, CollisionCircleSize);
   }
   
   void boundingBox(java.awt.geom.Rectangle2D r)
   {
      r.setRect(x - CollisionCircleSize, y - CollisionCircleSize, 
            2 * CollisionCircleSize, 2 * CollisionCircleSize);
   }

   void doMove(long now, GameMap map) 
   {
      if (hitpoints <= 0)
      {
         map.removeGameObject(this);
         GameObject health = new HealthPotionObject(cachedEngine, x, y); 
         map.addGameObject(health);

         return;
      }

      if (isAttacking)
      {
         if (now - lastMoveTime < AIM_TIME)
            isAiming = true;
         else
         {
            isAiming = false;
            if (now - lastMoveTime > ATTACK_TIME + AIM_TIME)
            {
               // we've extended bow long enough, fire the bow
               isAttacking = false;
               lastMoveTime = -1;
               GameObject arrow = new ArrowObject(cachedEngine, 
                     x + Math.cos(attackAngle) * 103, 
                     y + Math.sin(attackAngle) * 103,
                     attackAngle, now);
               map.addGameObject(arrow);
               shootClip.play();
            }
         }
      }

      if (!isAttacking)
      {
         double desiredAngle = Math.atan2(map.player.y - y, map.player.x - x);
         
         if (lastMoveTime < 0)
            lastMoveTime = now;

         if (now - lastMoveTime > TURN_TIME)
         {
            double diffangle = Util.constrain(desiredAngle - moveAngle);
            if (diffangle > - TURN_ANGLE && diffangle < TURN_ANGLE)
            {
               moveAngle = desiredAngle;

               // if we aren't attacking, see if we should attack
               if (!isAttacking && Util.dist(map.player.x, map.player.y, x, y) < attackDist)
               {
                  isAttacking = true;
                  attackAngle = desiredAngle;
                  lastMoveTime = now;
                  return;
               }
               
               if (now - lastMoveTime > LEG_STEP_TIME)
               {
                  pos++;
                  if (pos >= LEG_NUM)
                     pos = LEG_CENTER;
                  
                  lastMoveTime = lastMoveTime + LEG_STEP_TIME;
                  if (now - lastMoveTime > LEG_STEP_TIME)
                     lastMoveTime = now - LEG_STEP_TIME;

                double minMove = 0;
                double maxMove = MoveAmount;
                while(maxMove - minMove > 0.5)
                {
                   double moveStep = (minMove + maxMove) / 2;
                   double newx = x + Math.cos(moveAngle) * moveStep;
                   double newy = y + Math.sin(moveAngle) * moveStep;
                   if (!map.checkCollision(new CircleCollisionChecker(this, newx, newy, CollisionCircleSize)))
                      minMove = moveStep;
                   else
                      maxMove = moveStep;
                }
                Rectangle2D r = new Rectangle2D.Double();
                boundingBox(r);
                map.removeFromCollisionRegions(this, r);
                x = x + Math.cos(moveAngle) * minMove;
                y = y + Math.sin(moveAngle) * minMove;
                boundingBox(r);
                map.addToCollisionRegions(this, r);
               }
            }
            else
            {
               if (diffangle < 0) moveAngle -= TURN_ANGLE; else moveAngle += TURN_ANGLE;
               lastMoveTime = lastMoveTime + TURN_TIME;
               if (now - lastMoveTime > TURN_TIME)
                  lastMoveTime = now - TURN_TIME;
            }
         }
         
      }
         
   }

   void doDamage(int damage) {hitpoints -= damage;}
   boolean isEnemy() { return true; }
}
