/*
 * Created on Aug 2, 2004
 *
 * To change this generated comment go to 
 * Window>Preferences>Java>Code Generation>Code Template
 */
package org.programmingbasics.iu.game;

import java.awt.Graphics2D;
import java.awt.geom.Rectangle2D;
import java.util.HashSet;
import java.util.Iterator;

/**
 * @author The World
 */
public class EnemyBlob extends GameObjectWithPos {
   static final double CollisionCircleSize = 54;
   static final double CollisionAttackCircleSize = 59;
   static RotatingSprite blobImg;
   int hitpoints = 2;
   double moveAngle;
   long moveTime = -1;
   long lastTime = -1;
   boolean hasAttacked = false;
   static final long MAX_MOVE_TIME = 500;
   static final long MOVE_RESET = -3000;
   static final long MOVE_TIME = 1000;
   static final double MOVE_SIZE = 0.1;
   static final double HEALTH_DROP_PERCENTAGE = 0.3;

   GameEngine cachedEngine;
   
   /**
    * 
    */
   public EnemyBlob(GameEngine e, double x, double y) {
      super();
      // TODO: NOT SYNCHRONIZED!
      if (blobImg == null)
         blobImg = new RotatingSprite(e.loadImage("blob1.png"), 54, 54);
      cachedEngine = e;
      this.x = x;
      this.y = y;
   }

   void draw(GameEngine e, Graphics2D g)
   {
      e.drawSprite(blobImg, x, y, 0);
   }

   boolean checkCollision(CollisionChecker check)
   {
      return check.checkCircleCollision(x, y, CollisionCircleSize);
   }
   
   void boundingBox(java.awt.geom.Rectangle2D r)
   {
      r.setRect(x - CollisionCircleSize, y - CollisionCircleSize, 
            2 * CollisionCircleSize, 2 * CollisionCircleSize);
   }

   void doMove(long now, GameMap map) 
   {
      if (hitpoints <= 0)
      {
         map.removeGameObject(this);
         if (Math.random() < HEALTH_DROP_PERCENTAGE)
         {
            GameObject health = new HealthPotionObject(cachedEngine, x, y); 
            map.addGameObject(health);
         }
         return;
      }
      long elapsed = now - lastTime;
      long timeSpentMoving = elapsed;
      lastTime = now;
      if (moveTime < 0 && moveTime + elapsed >= 0)
         moveAngle = Math.atan2(map.player.y - y, map.player.x - x);
      moveTime += elapsed;
      
      if (timeSpentMoving > moveTime)
         timeSpentMoving = moveTime;
      if (timeSpentMoving > MAX_MOVE_TIME)
         timeSpentMoving = MAX_MOVE_TIME;
      if (moveTime > MOVE_TIME)
      {
         hasAttacked = false;
         moveTime = MOVE_RESET;
      }
      
      if (timeSpentMoving > 0)
      {
         double minMove = 0;
         double maxMove = timeSpentMoving * MOVE_SIZE;
         while(maxMove - minMove > 0.5)
         {
            double moveStep = (minMove + maxMove) / 2;
            double newx = x + Math.cos(moveAngle) * moveStep;
            double newy = y + Math.sin(moveAngle) * moveStep;
            if (!map.checkCollision(new CircleCollisionChecker(this, newx, newy, CollisionCircleSize)))
               minMove = moveStep;
            else
               maxMove = moveStep;
         }
         Rectangle2D r = new Rectangle2D.Double();
         boundingBox(r);
         map.removeFromCollisionRegions(this, r);
         x = x + Math.cos(moveAngle) * minMove;
         y = y + Math.sin(moveAngle) * minMove;
         boundingBox(r);
         map.addToCollisionRegions(this, r);
         
         if (!hasAttacked)
         {
            HashSet s = new HashSet();
            map.findAllCollisions(new CircleCollisionChecker(this, x, y, CollisionAttackCircleSize), s);
            Iterator it = s.iterator();
            while(it.hasNext())
            {
               GameObject o = (GameObject)it.next();
               if (o.isPlayer())
               {
                  hasAttacked = true;
                  o.doDamage(5);
               }
            }
         }
      }
         
   }

   void doDamage(int damage) {hitpoints -= damage;}
   boolean isEnemy() { return true; }
}
