/*
 * Created on Jun 22, 2004
 *
 * To change this generated comment go to 
 * Window>Preferences>Java>Code Generation>Code Template
 */
package org.programmingbasics.iu.game;

import java.awt.Graphics2D;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseMotionAdapter;
import java.awt.font.FontRenderContext;
import java.awt.font.LineBreakMeasurer;
import java.awt.font.TextAttribute;
import java.awt.font.TextLayout;
import java.awt.geom.AffineTransform;
import java.awt.image.BufferedImage;
import java.io.IOException;
import java.io.InputStream;
import java.text.AttributedString;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Vector;

import javax.imageio.ImageIO;

/**
 * @author The World
 */
public class GameEngine
{
   GameWindow gw;
   GameScreen curgs;
   public RenderBuffer render;
   public UserEventQueue events;

   class DelayedCenteredText
   {
      double x, y, width;
      String text;
   }
   Vector delayedText = new Vector();
   // Add text that will be drawn later (so that it is on a different layer than sprites)
   public void addDelayedCenteredText(double x, double y, double width, String text)
   {
      DelayedCenteredText dct = new DelayedCenteredText();
      dct.x = x;
      dct.y = y;
      dct.width = width;
      dct.text = text;
      delayedText.add(dct);
   }
   public void drawDelayedText(Graphics2D g)
   {
      Iterator it = delayedText.iterator();
      while (it.hasNext())
      {
         DelayedCenteredText dct = (DelayedCenteredText)it.next();
         
         AffineTransform oldTransform = g.getTransform();
         g.translate(dct.x, dct.y);
         g.scale(1, -1);
         g.translate(-dct.x, -dct.y);
         
         
         Hashtable map = new Hashtable();
         map.put(TextAttribute.FONT, g.getFont());
         FontRenderContext frc = g.getFontRenderContext();
         AttributedString styledText = new AttributedString(dct.text, map);
         LineBreakMeasurer measurer = new LineBreakMeasurer(styledText.getIterator(), frc);

         float texty = (float)dct.y;
         while (true) 
         {
             TextLayout layout = measurer.nextLayout((float)dct.width);
             if (layout == null) break;

             texty += (layout.getAscent());
             float textx = (float)dct.x - (layout.isLeftToRight() ? 1 : -1) * 
                  layout.getAdvance() / 2;
             layout.draw(g, textx, texty);
             texty += layout.getDescent() + layout.getLeading();
         }

         g.setTransform(oldTransform);
      }
      delayedText.clear();
   }
   
   public void drawCenteredText(Graphics2D g, double x, double y, double width, String text)
   {
      AffineTransform oldTransform = g.getTransform();
      g.translate(x, y);
      g.scale(1, -1);
      g.translate(-x, -y);
      
      
      Hashtable map = new Hashtable();
      map.put(TextAttribute.FONT, g.getFont());
      FontRenderContext frc = g.getFontRenderContext();
      AttributedString styledText = new AttributedString(text, map);
      LineBreakMeasurer measurer = new LineBreakMeasurer(styledText.getIterator(), frc);

      float texty = (float)y;
      while (true) 
      {
          TextLayout layout = measurer.nextLayout((float)width);
          if (layout == null) break;

          texty += (layout.getAscent());
          float textx = (float)x - (layout.isLeftToRight() ? 1 : -1) * 
               layout.getAdvance() / 2;
          layout.draw(g, textx, texty);
          texty += layout.getDescent() + layout.getLeading();
      }

      g.setTransform(oldTransform);
   }
   
   public void drawSprite(RotatingSprite s, double x, double y, double angle)
   {
      Graphics2D g = render.getGraphics();
      AffineTransform t = new AffineTransform();
      t.translate(x, y);
      t.rotate(angle);
      t.scale(1, -1);  // switching from world to screen coordinates inverts the image, this inverts the image back
      t.translate(-s.centerx, -s.centery);
      g.drawRenderedImage(s.img, t);
   }

   public void switchBuffer()
   {
      render.switchBuffer();
      gw.repaint();
   }
   
   public GameEngine(GameWindow gw)
   {
      super();
      this.gw = gw;
      render = new RenderBuffer(gw, 550, 550);
      events = new UserEventQueue();
      gw.setSize(100, 100);
      gw.addKeyListener(new KeyAdapter()
         {
            public void keyPressed(KeyEvent e) 
            {
               events.keyPress(e.getKeyCode());
            }
            public void keyReleased(KeyEvent e) 
            {
               events.keyRelease(e.getKeyCode());
            }
            
         });
      gw.addMouseListener(new MouseAdapter()
         {
            public void mousePressed(MouseEvent e) 
            {
               if (e.getButton() == MouseEvent.BUTTON1)
                  events.mousePress(e.getX(), e.getY());
            }

            public void mouseReleased(MouseEvent e) 
            {
               if (e.getButton() == MouseEvent.BUTTON1)
                  events.mouseRelease(e.getX(), e.getY());
            }
            
         });
      gw.addMouseMotionListener(new MouseMotionAdapter()
         {
            public void mouseDragged(MouseEvent e) 
            {
               events.mouseMove(e.getX(), e.getY());
            }

            public void mouseMoved(MouseEvent e) 
            {
               events.mouseMove(e.getX(), e.getY());
            }
         });
         
      setGameScreen(new PlayScreen(this));
   }

   // Remember to close the stream after use
   public InputStream loadResource(String name)
   {
      ClassLoader cl = this.getClass().getClassLoader();
      try {
         return cl.getResource("resources/" + name).openStream();
      } catch (IOException e) {
         e.printStackTrace();
      }
      return null;
   }
   
   public BufferedImage loadImage(String name)
   {
      ClassLoader cl = this.getClass().getClassLoader();
      // TODO: replace null with an empty image?
      BufferedImage img = null; 
      try
      {
         img = ImageIO.read(cl.getResource("resources/" + name));
      }
      catch (IOException e)
      {
         e.printStackTrace();
      }
      return img;
   }

   public SoundClip loadSound(String name)
   {
      ClassLoader cl = this.getClass().getClassLoader();
      // TODO: replace null with an empty clip?
      SoundClip clip = null; 
      clip = new SoundClip(cl.getResource("resources/" + name));
      return clip;
   }

   public void setGameScreen(GameScreen gs)
   {
      curgs = gs;
      gw.gs = gs;
   }

   void run()
   {
      curgs.run();
   }

}
