/*
 * Created on Jul 6, 2004
 *
 * To change this generated comment go to 
 * Window>Preferences>Java>Code Generation>Code Template
 */
package org.programmingbasics.iu.game;

import java.awt.Graphics2D;
import java.awt.geom.Rectangle2D;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.io.StreamTokenizer;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Vector;

/**
 * @author The World
 */
public class GameMap {
   public int [][] mapTiles = new int[80][80];
   public Vector [][] collisionRegions = new Vector[80][80];
   public Vector gameObjects = new Vector();
   public Player player;
   RotatingSprite [] tileImgs = new RotatingSprite[5];
   int tileSize = 128;
   public boolean lose = false;
   public boolean win = false;
   public int fastWalk = 0;
   public boolean showPos = false;
   public boolean noMove = false;
   
   class Tokenizer
   {
      StreamTokenizer tok;
      Tokenizer(InputStream stream)
      {
         tok = new StreamTokenizer(stream);
         tok.resetSyntax();
         tok.eolIsSignificant(false);
         tok.lowerCaseMode(false);
         tok.wordChars('a', 'z');
         tok.wordChars('A', 'Z');
         tok.wordChars('0', '9');
         tok.wordChars('.', '.');
         tok.wordChars(128 + 32, 255);
         tok.whitespaceChars(0, ' ');
         tok.commentChar('/');
         tok.quoteChar('"');
      }
      String next()
      {
         try {
            int toktype = tok.nextToken();
            if (toktype == StreamTokenizer.TT_EOF) return null;
            if (toktype == '"') return tok.sval;
            if (toktype > 0) return String.valueOf((char)toktype);
            if (toktype != StreamTokenizer.TT_WORD) return null;
         } catch (IOException e) {
            return null;
         }
         return tok.sval;
      }
   }

   public GameMap(GameEngine engine)
   {
      tileImgs[0] = new RotatingSprite(engine.loadImage("grass.png"), 0, 0);
      tileImgs[1] = new RotatingSprite(engine.loadImage("cobblestone.png"), 0, 0);
      tileImgs[2] = new RotatingSprite(engine.loadImage("dirt.png"), 0, 0);
      tileImgs[3] = new RotatingSprite(engine.loadImage("deadgrass.png"), 0, 0);
      tileImgs[4] = new RotatingSprite(engine.loadImage("water.png"), 0, 0);
      InputStream mapStream = engine.loadResource("map");
/*
      try {
         mapStream = new FileInputStream("map");
      } catch (FileNotFoundException e) {}
*/      
      Tokenizer tok = new Tokenizer(mapStream);
      int maprow = 0;
      String semicolon;
      while(true)
      {
         String w = tok.next();
         if (w == null) break;
         if (w.equals("mapdim"))
         {
            int width = Integer.parseInt(tok.next());
            int height = Integer.parseInt(tok.next());
            mapTiles = new int[width][height];
            collisionRegions = new Vector[width][height];
            for (int y = 0; y < height; y++)
               for (int x = 0; x < width; x++)
                  collisionRegions[x][y] = new Vector();
            semicolon = tok.next();
         }
         else if (w.equals("map"))
         {
            String line = tok.next();
            for (int x = 0; x < line.length(); x++)
            {
               int ch = line.charAt(x);
               switch(ch)
               {
                  case '0':
                  case '1':
                  case '2':
                  case '3':
                  case '4':
                     mapTiles[x][maprow] = ch - '0';
                     break;
               }
            }
            maprow++;
            semicolon = tok.next();
         }
         else if (w.equals("tree"))
         {
            double x = Double.parseDouble(tok.next());
            double y = Double.parseDouble(tok.next());
            boolean isDead = (mapTiles[(int)(x / tileSize)][(int)(y / tileSize)] == 3);
            addGameObject(new TreeObject(engine, x, y, isDead));
            semicolon = tok.next();
            
         }
         else if (w.equals("blob"))
         {
            double x = Double.parseDouble(tok.next());
            double y = Double.parseDouble(tok.next());
            addGameObject(new EnemyBlob(engine, x, y));
            semicolon = tok.next();
            
         }
         else if (w.equals("wall"))
         {
            double x = Double.parseDouble(tok.next());
            double y = Double.parseDouble(tok.next());
            addGameObject(new WallObject(engine, x, y));
            semicolon = tok.next();
            
         }
         else if (w.equals("hwall"))
         {
            double x = Double.parseDouble(tok.next());
            double y = Double.parseDouble(tok.next());
            int count = Integer.parseInt(tok.next());
            for (int n = 0; n < count; n++) 
               addGameObject(new WallObject(engine, x + 50 * n, y));
            semicolon = tok.next();
         }
         else if (w.equals("vwall"))
         {
            double x = Double.parseDouble(tok.next());
            double y = Double.parseDouble(tok.next());
            int count = Integer.parseInt(tok.next());
            for (int n = 0; n < count; n++) 
               addGameObject(new WallObject(engine, x, y + 50 * n));
            semicolon = tok.next();
         }
         else if (w.equals("house"))
         {
            double x = Double.parseDouble(tok.next());
            double y = Double.parseDouble(tok.next());
            int width = Integer.parseInt(tok.next());
            int height = Integer.parseInt(tok.next());
            String opening = tok.next();
            for (int n = 0; n <= width; n++)
               if (!opening.equals("s") || n - width / 2 < -1 || n - width / 2 > 1)
                  addGameObject(new WallObject(engine, x + 50 * n, y));
            for (int n = 0; n <= width; n++) 
               if (!opening.equals("n") || n - width / 2 < -1 || n - width / 2 > 1)
                  addGameObject(new WallObject(engine, x + 50 * n, y + height * 50));
            for (int n = 1; n < height; n++) 
               if (!opening.equals("w") || n - height / 2 < -1 || n - height / 2 > 1)
                  addGameObject(new WallObject(engine, x, y + 50 * n));
            for (int n = 1; n < height; n++) 
               if (!opening.equals("e") || n - height / 2 < -1 || n - height / 2 > 1)
                  addGameObject(new WallObject(engine, x + width * 50, y + 50 * n));
            semicolon = tok.next();
         }
         else if (w.equals("villager"))
         {
            int leader = Integer.parseInt(tok.next());
            double x = Double.parseDouble(tok.next());
            double y = Double.parseDouble(tok.next());
            double angle = Double.parseDouble(tok.next());
            String statement = tok.next();
            addGameObject(new VillagerObject(engine, leader, x, y, angle, statement));
            semicolon = tok.next();
         }
         else if (w.equals("archer"))
         {
            double x = Double.parseDouble(tok.next());
            double y = Double.parseDouble(tok.next());
            addGameObject(new EnemyArcher(engine, x, y));
            semicolon = tok.next();
            
         }
         else if (w.equals("crystal"))
         {
            double x = Double.parseDouble(tok.next());
            double y = Double.parseDouble(tok.next());
            addGameObject(new WinningCrystalObject(engine, x, y));
            semicolon = tok.next();
            
         }
      }
   }

   void addGameObject(GameObject obj)
   {
      gameObjects.add(obj);
      
      // add to Collision regions
      Rectangle2D r = new Rectangle2D.Double();
      obj.boundingBox(r);
      addToCollisionRegions(obj, r);
   }

   void removeGameObject(GameObject obj)
   {
      // remove from Collision regions
      Rectangle2D r = new Rectangle2D.Double();
      obj.boundingBox(r);
      removeFromCollisionRegions(obj, r);

      gameObjects.remove(obj);
   }

   void addToCollisionRegions(final GameObject obj, Rectangle2D r)
   {
      class AddCollide extends CollisionRegionClosure
      {
         boolean handleVector(int x, int y, Vector region) 
         {
            region.add(obj);
            return true;
         }
      }
      iterateOverCollisionRegions(r, new AddCollide());
   }

   void removeFromCollisionRegions(final GameObject obj, Rectangle2D r)
   {
      class RemoveCollide extends CollisionRegionClosure
      {
         boolean handleVector(int x, int y, Vector region) 
         {
            region.remove(obj);
            return true;
         }
      }
      iterateOverCollisionRegions(r, new RemoveCollide());
   }
   
   void draw(final GameEngine engine, final Graphics2D g, double centerx, double centery, double width, double height)
   {
      double left = centerx - width / 2;
      double bottom = centery - height / 2;
      
      int leftTile = (int)Math.floor(left / tileSize);
      int bottomTile = (int)Math.floor(bottom / tileSize);
      int widthInTiles = (int)Math.ceil(width / tileSize) + 2;
      int heightInTiles = (int)Math.ceil(height / tileSize) + 2;
      
      for (int x = leftTile; x < leftTile + widthInTiles; x++)
      {
         if (x < 0 || x >= mapTiles.length) continue;
         for (int y = bottomTile; y < bottomTile + heightInTiles; y++)
         {
            if (y < 0 || y >= mapTiles[x].length) continue;
            engine.drawSprite(tileImgs[mapTiles[x][y]], x * tileSize, y * tileSize + tileSize, 0);
         }
      }
      
      class DrawCollide extends CollisionRegionClosure
      {
         HashSet set = new HashSet();
         boolean handle(GameObject o) {
            if (!set.contains(o))
            {
               set.add(o);
               o.draw(engine, g);
            }
            return true;
         }
      }
      Rectangle2D r = new Rectangle2D.Double(left, bottom, 
            widthInTiles * tileSize, heightInTiles * tileSize); 
      iterateOverCollisionRegions(r, new DrawCollide());

      class PosCollide extends CollisionRegionClosure
      {
         HashSet set = new HashSet();
         boolean handle(GameObject o) {
            if ((o instanceof GameObjectWithPos) && !set.contains(o))
            {
               set.add(o);
               GameObjectWithPos posobj = (GameObjectWithPos)o;
               engine.addDelayedCenteredText(posobj.x, posobj.y, 100, "" + posobj.x + "," + posobj.y);
            }
            return true;
         }
      }
      if (showPos)
         iterateOverCollisionRegions(r, new PosCollide());

   }

   class CollisionRegionClosure
   {
      // Return false to stop iterating
      boolean handleVector(int x, int y, Vector objs)
      {
         Iterator it = objs.iterator();
         while (it.hasNext())
         {
            GameObject o = (GameObject)it.next();
            if (!handle(o)) return false;
         }
         return true;
      }
      boolean handle(GameObject o) {return true;}
   }
   
   void move(GameEngine engine, final long now, double centerx, double centery)
   {
      if (noMove)
      {
         player.doMove(now, this);
         return;
      }
      
      double left = centerx - engine.render.getWidth() / 2 - tileSize * 3;
      double bottom = centery - engine.render.getHeight() / 2 - tileSize * 3;
      
      int widthInTiles = (int)Math.ceil(engine.render.getWidth() / tileSize) + 6;
      int heightInTiles = (int)Math.ceil(engine.render.getHeight() / tileSize) + 6;
      
      class MoveCollide extends CollisionRegionClosure
      {
         HashSet set = new HashSet();
         boolean handle(GameObject o) {
            if (!set.contains(o))
            {
               set.add(o);
            }
            return true;
         }
      }
      
      Rectangle2D r = new Rectangle2D.Double(left, bottom, 
            widthInTiles * tileSize, widthInTiles * tileSize);
      MoveCollide movingObjs = new MoveCollide();
      iterateOverCollisionRegions(r, movingObjs);
      Iterator it = movingObjs.set.iterator();
      while (it.hasNext())
      {
         GameObject o = (GameObject)it.next();
         o.doMove(now, GameMap.this);
      }
   }

   void iterateOverCollisionRegions(Rectangle2D r, CollisionRegionClosure closure)
   {
      int ulx = Math.max((int)Math.floor(r.getMinX() / tileSize), 0);
      int uly = Math.max((int)Math.floor(r.getMinY() / tileSize), 0);
      int lrx = Math.min((int)Math.ceil(r.getMaxX() / tileSize), collisionRegions.length - 1);
      int lry = Math.min((int)Math.ceil(r.getMaxY() / tileSize), collisionRegions[0].length - 1);
      for (int xx = ulx; xx <= lrx; xx++)
      {
         for (int yy = uly; yy <= lry; yy++)
         {
            if (!closure.handleVector(xx, yy, collisionRegions[xx][yy]))
               return;
         }
      }
   }
   
   boolean checkCollision(final CollisionChecker check)
   {
      if (check.checkLineCollision(0, 0, 0, mapTiles[0].length * tileSize)) return true;
      if (check.checkLineCollision(0, 0, mapTiles.length * tileSize, 0)) return true;
      if (check.checkLineCollision(0, mapTiles[0].length * tileSize, 
            mapTiles.length * tileSize, mapTiles[0].length * tileSize)) return true;
      if (check.checkLineCollision(mapTiles.length * tileSize, 0, 
            mapTiles.length * tileSize, mapTiles[0].length * tileSize)) return true;

      Rectangle2D r = new Rectangle2D.Double();
      check.boundingBox(r);
      class CollideCheck extends CollisionRegionClosure
      {
         boolean hit = false;
         boolean handle(GameObject o) 
         {
            if (check.checkCollision(o))
            {
               hit = true;
               return false;
            }
            return true;
         }
      }
      CollideCheck c = new CollideCheck();
      iterateOverCollisionRegions(r, c);
      return c.hit;
   }

   void findAllCollisions(final CollisionChecker check, final HashSet collided)
   {
      Rectangle2D r = new Rectangle2D.Double();
      check.boundingBox(r);
      class FindCollide extends CollisionRegionClosure
      {
         boolean handle(GameObject o) 
         {
            if (check.checkCollision(o))
               collided.add(o);
            return true;
         }
      }
      FindCollide f = new FindCollide();
      iterateOverCollisionRegions(r, f);
   }

}
